<?php

/**
 * Whois server list.
 */
include_once( 'class-whoisservers.php' );

/**
 * Class for performing whois queries.
 */
class Domaincheck {

	/**
	 * Query a whois server.
	 * 
	 * @param string $domain 
	 * @param string $tld 
	 * @param string $whoisserver 
	 * @return string Whois entry.
	 */
	private static function whois( $domain, $tld, $whoisserver ) {
		try {
			// use own error handler to handle also warnings in fsockopen
			set_error_handler( function ( $severity, $message, $file, $line ) {
				throw new ErrorException( $message, 0, $severity, $file, $line );
			});
	 		$fp = fsockopen( $whoisserver, 43, $errno, $errstr, 30 );
	 		// reset error handler function
			restore_error_handler();
			if ( ! $fp )
				return 'whois_query_error' . $errstr .' (' . $errno . ')';
			else {
				if ( 'de' != $tld && ! preg_match( '/^[a-zA-Z0-9-]*$/', $domain ) ) {
					// use idna class to encode idn domain
					require_once( 'class-idna-convert.php' );
					$idn = new idna_convert();
					$domain = $idn->encode( $domain );
				}
				if ( ! preg_match( '/^[a-zA-Z0-9\.]*$/', $tld ) ) {
					// use idna class to encode idn tld
					require_once( 'class-idna-convert.php' );
					$idn = new idna_convert();
					$tld = $idn->encode( $tld );
				}
				$flag = '';
				if ( 'de' == $tld )
					$flag = '-T dn ';
				fwrite( $fp, $flag . $domain . '.' . $tld . "\r\n" );
				$string = '';
				while( ! feof( $fp ) )
					$string .= fread( $fp, 128 );
				fclose( $fp );

				return trim( $string );
			}
 		} catch ( Exception $e ) {
			return 'whois_query_error' . $e->getMessage();
		} finally {
			// reset error handler function
			restore_error_handler();
		}
	}

	/**
	 * Evaluate whois query result.
	 * 
	 * @param string $domain 
	 * @param string $tld 
	 * @return string Registration status.
	 */
	public static function get_whois_result( $domain, $tld ) {
		$json_result = array();
		$json_result['domain'] = $domain;
		$json_result['tld'] = $tld;

		if ( ! preg_match( '/^[^_\.\/]{1,}$/', $domain ) )
			$json_result['status'] = 'invalid';
		else {
			$whoisserver = WP24_Domain_Check_Whoisservers::get_whoisserver( $tld );
			if ( ! $whoisserver )
				$json_result['status'] = 'whoisserver';
			else
			{
				$string = self::whois( $domain, $tld, $whoisserver['host'] );
				if ( preg_match( '/whois_query_error/', $string ) ) {
					$json_result['status'] = 'error';
					$json_result['text'] = str_replace( 'whois_query_error', '', $string );
				}
				else {
					if ( preg_match( '/' . $whoisserver['free'] . '/i', preg_replace( '/\s\s+|\t/', ' ', $string ) ) )
						$json_result['status'] = 'available';
					else if ( preg_match( '/' . WP24_Domain_Check_Whoisservers::get_pattern_invalid() . '/i', preg_replace( '/\s\s+|\t/', ' ', $string ) ) )
						$json_result['status'] = 'invalid';
					else if ( preg_match( '/' . WP24_Domain_Check_Whoisservers::get_pattern_limit() . '/i', $string ) )
						$json_result['status'] = 'limit';
					else {
						$json_result['status'] = 'registered';
						$json_result['text'] = $string;
					}
				}
			}
		}
		return json_encode( $json_result, JSON_INVALID_UTF8_IGNORE );
	}

}
